#
# Rules and settings applicable to all targets
#
# Include target-specific settings, if they exist. First, look in the current
# directory, then in the targets directory, and finally in targets/unofficial.
# If none of these exist, there are special rule files for Intel Linux.
ifneq ($(wildcard $(HALCONARCH).mak),)
  TARGET_MAKEFILE:=$(HALCONARCH).mak
else ifneq ($(wildcard targets/$(HALCONARCH).mak),)
  TARGET_MAKEFILE:=targets/$(HALCONARCH).mak
else ifneq ($(wildcard targets/unofficial/$(HALCONARCH).mak),)
  TARGET_MAKEFILE:=targets/unofficial/$(HALCONARCH).mak
else ifneq ($(filter x86sse2-linux2.4-gcc%,$(HALCONARCH)),)
  TARGET_MAKEFILE:=targets/x86sse2-linux.mak
else ifneq ($(filter x64-linux2.4-gcc%,$(HALCONARCH)),)
  TARGET_MAKEFILE:=targets/x64-linux.mak
else ifneq ($(filter x64-linux,$(HALCONARCH)),)
  TARGET_MAKEFILE:=targets/x64-linux.mak
endif

ifneq ($(TARGET_MAKEFILE),)
# $(info Including extra definitions from $(TARGET_MAKEFILE))
  include $(TARGET_MAKEFILE)
endif

# Include rules specific to a compiler family.
ifneq ($(filter gcc%,$(TARGET_COMPILER)), )
  include rules/gcc.mak
else ifeq ($(TARGET_COMPILER), cl)
  include rules/cl.mak
else ifneq ($(filter cg%,$(TARGET_COMPILER)), )
  include rules/cgtools.mak
else
  $(info Unknown compiler $(TARGET_COMPILER); assuming it is gcc like)
  include rules/gcc.mak
endif

# Look for C/C++ source files in the $(GENDIR) directory as well as the
# current directory.
vpath %.c               $(GENDIR)
vpath %.cc              $(GENDIR)
vpath %.cpp             $(GENDIR)

# Handle debug/tuning/release builds
ifeq ($(STATE),debug)
CFLAGS                 += $(call debug,$(TARGET_DEBUGFLAGS))
CXXFLAGS               += $(call debug,$(TARGET_DEBUGFLAGS))
LDFLAGS                += $(call ldebug,$(TARGET_LDFLAGS))
LXXFLAGS               += $(call ldebug,$(TARGET_LXXFLAGS))
else ifeq ($(STATE),tuning)
# Use the optimization flags for tuning builds, plus compiler specific flags
# for profiling. Note that the compiler specific rules must make sure there
# are no incompatible flags.
CFLAGS                 += $(call profile,$(TARGET_OPTFLAGS))
CXXFLAGS               += $(call profile,$(TARGET_OPTFLAGS))
LDFLAGS                += $(call lprofile,$(TARGET_LDFLAGS))
LXXFLAGS               += $(call lprofile,$(TARGET_LXXFLAGS))
else
CPPFLAGS               += $(call adddef,_NDEBUG)
CPPXXFLAGS             += $(call adddef,_NDEBUG)
CFLAGS                 += $(TARGET_OPTFLAGS)
CXXFLAGS               += $(TARGET_OPTFLAGS)
LDFLAGS                += $(TARGET_LDFLAGS)
LXXFLAGS               += $(TARGET_LXXFLAGS)
endif

# Add target specific flags to flag variables
AFLAGS                 += $(TARGET_AFLAGS)
CPPFLAGS               += $(TARGET_CPPFLAGS)
CPPXXFLAGS             += $(TARGET_CPPXXFLAGS)
CFLAGS                 += $(TARGET_CFLAGS)
CXXFLAGS               += $(TARGET_CXXFLAGS)
LDLIBS                 += $(TARGET_LDLIBS)
LXXLIBS                += $(TARGET_LXXLIBS)

# Generate the list of dependency files automatically from the source files.
depfromsrc              = $(addprefix $(GENPATH),\
                                      $(patsubst %.c,%.d,\
                                                 $(filter %.c,$(notdir $(1))))\
                                      $(patsubst %.cpp,%.d,\
                                                 $(filter %.cpp,$(notdir $(1))))\
                                      $(patsubst %.cc,%.d,\
                                                 $(filter %.cc,$(notdir $(1)))))

DEPENDENCIES           ?= $(call depfromsrc,$(SOURCES))

# Generate the list of object files automatically from the source files.
objfromsrc              = $(addprefix $(OBJPATH),\
                                      $(patsubst %.c,%$(OBJ),\
                                                 $(filter %.c,$(notdir $(1))))\
                                      $(patsubst %.cpp,%$(OBJ),\
                                                 $(filter %.cpp,$(notdir $(1)))))

OBJECTS                ?= $(call objfromsrc,$(SOURCES))

# Commands to compile C and C++ source code using the standard macros. If
# used via the 'call' command, extra parameters can be passed.
CCCMD  = $(CC) $(CPPFLAGS) $(1) $(CFLAGS) $(2) $(call objout,$@) $<
CXXCMD = $(CXX) $(CPPXXFLAGS) $(1) $(CXXFLAGS) $(2) $(call objout,$@) $<

# Commands to link C and C++ executables using the standard macros. If used
# via the 'call' command, extra parameters can be passed.
LDCMD ?= $(LD) $(LDFLAGS) $(1) $(filter %$(OBJ),$^) $(2) $(LDLIBS) $(3) $(call exeout,$@)
LXXCMD ?= $(LXX) $(LXXFLAGS) $(1) $(filter %$(OBJ),$^) $(2) $(LXXLIBS) $(3) $(call exeout,$@)

# Common targets and their rules.
.PHONY: all clean depend distclean doc generated tags veryclean

all: generated $(TARGETS)

generated: $(GENERATED)

# Create doxygen documentation. Will likely only work on Unix build
# platforms.
doc: $(HEADERS) $(SOURCES)
	doxygen Doxyfile $<

# Create ctags. Will likely only work on Unix build platforms.
tags: $(HEADERS) $(SOURCES)
	ctags $<

# 'clean' will remove all build products for the current target and build
# state. 'veryclean' will also remove all build product directories for the
# current target (for all build states). 'distclean' will remove all
# build product directories for all targets and states. Each of these is a
# '::' rules so that additions can be made if required.
# Note that it is assumes *any* file contained in any of the build product
# directories is a build product.
ifeq ($(OS),Windows_NT)

clean::
	@-if exist $(GENDIR)$(DIRSEP)* ( $(RM) $(GENDIR)$(DIRSEP)* )
	@-if exist $(OBJDIR)$(DIRSEP)* ( $(RM) $(OBJDIR)$(DIRSEP)* )
	@-if exist $(LIBDIR)$(DIRSEP)* ( $(RM) $(LIBDIR)$(DIRSEP)* )
	@-if exist $(BINDIR)$(DIRSEP)* ( $(RM) $(BINDIR)$(DIRSEP)* )

veryclean::	clean
	@-if exist bin$(DIRSEP)$(HALCONARCH) $(RMDIR) bin$(DIRSEP)$(HALCONARCH)
	@-if exist bind$(DIRSEP)$(HALCONARCH) $(RMDIR) bind$(DIRSEP)$(HALCONARCH)
	@-if exist bint$(DIRSEP)$(HALCONARCH) $(RMDIR) bint$(DIRSEP)$(HALCONARCH)
	@-if exist gen$(DIRSEP)$(HALCONARCH) $(RMDIR) gen$(DIRSEP)$(HALCONARCH)
	@-if exist gend$(DIRSEP)$(HALCONARCH) $(RMDIR) gend$(DIRSEP)$(HALCONARCH)
	@-if exist gent$(DIRSEP)$(HALCONARCH) $(RMDIR) gent$(DIRSEP)$(HALCONARCH)
	@-if exist lib$(DIRSEP)$(HALCONARCH) $(RMDIR) lib$(DIRSEP)$(HALCONARCH)
	@-if exist libd$(DIRSEP)$(HALCONARCH) $(RMDIR) libd$(DIRSEP)$(HALCONARCH)
	@-if exist libt$(DIRSEP)$(HALCONARCH) $(RMDIR) libt$(DIRSEP)$(HALCONARCH)
	@-if exist obj$(DIRSEP)$(HALCONARCH) $(RMDIR) obj$(DIRSEP)$(HALCONARCH)
	@-if exist objd$(DIRSEP)$(HALCONARCH) $(RMDIR) objd$(DIRSEP)$(HALCONARCH)
	@-if exist objt$(DIRSEP)$(HALCONARCH) $(RMDIR) objt$(DIRSEP)$(HALCONARCH)

distclean::	clean
	@-if exist bin $(RMDIR) bin
	@-if exist bind $(RMDIR) bind
	@-if exist bint $(RMDIR) bint
	@-if exist gen $(RMDIR) gen
	@-if exist gend $(RMDIR) gend
	@-if exist gent $(RMDIR) gent
	@-if exist lib $(RMDIR) lib
	@-if exist libd $(RMDIR) libd
	@-if exist libt $(RMDIR) libt
	@-if exist obj $(RMDIR) obj
	@-if exist objd $(RMDIR) objd
	@-if exist objt $(RMDIR) objt

else

clean::
	@-$(RM) $(GENDIR)$(DIRSEP)*
	@-$(RM) $(OBJDIR)$(DIRSEP)*
	@-$(RM) $(LIBDIR)$(DIRSEP)*
	@-$(RM) $(BINDIR)$(DIRSEP)*

veryclean::	clean
	@-$(RM) bin$(DIRSEP)$(HALCONARCH) bind$(DIRSEP)$(HALCONARCH) \
		bint$(DIRSEP)$(HALCONARCH)
	@-$(RM) gen$(DIRSEP)$(HALCONARCH) gend$(DIRSEP)$(HALCONARCH) \
		gent$(DIRSEP)$(HALCONARCH)
	@-$(RM) lib$(DIRSEP)$(HALCONARCH) libd$(DIRSEP)$(HALCONARCH) \
		libt$(DIRSEP)$(HALCONARCH)
	@-$(RM) obj$(DIRSEP)$(HALCONARCH) objd$(DIRSEP)$(HALCONARCH) \
		objt$(DIRSEP)$(HALCONARCH)

distclean::	clean
	@-$(RM) bin bind bint gen gend gent lib libd libt obj objd objt

endif

# 'depend' is a dummy target. The actual work is done by the mechanism used
# to automatically generate/include dependencies.
depend:: $(patsubst $(OBJDIR)%,$(GENDIR)%,$(OBJECTS:$(OBJ)=.d))
	@touch $(GENDIR)

$(BINDIR):
	@$(MKDIR) $(BINDIR)
$(GENDIR):
	@$(MKDIR) $(GENDIR)
$(LIBDIR):
	@$(MKDIR) $(LIBDIR)
$(OBJDIR):
	@$(MKDIR) $(OBJDIR)
