#
# Basic definitions and settings. Include this first.
#
ifeq ($(HALCONARCH),)
$(error The environment variable HALCONARCH must be set !)
else ifneq ($(filter %.mak,$(HALCONARCH)),)
$(error HALCONARCH should not end in .mak (probably a typo))
endif

# Now attempt to parse the HALCONARCH to determine the target CPU, vendor,
# operating system and which compiler to use. First, assume the
# 'CPU-OS-COMPILER' naming scheme used by standard HALCON.
TARGET_CPU             ?= $(word 1,$(subst -, ,$(HALCONARCH)))
TARGET_VENDOR          ?= unknown
TARGET_OS              ?= $(word 2,$(subst -, ,$(HALCONARCH)))
TARGET_COMPILER        ?= $(word 3,$(subst -, ,$(HALCONARCH)))

# Windows uses the 'CPU-OS' form. Assume the 'cl' compiler by default.
# This also applies to Windows CE.
ifneq ($(or $(filter win32,$(TARGET_OS)),$(filter win64,$(TARGET_OS)),\
            $(filter wince%,$(TARGET_OS))), )
  ifeq ($(TARGET_VENDOR), unknown)
    TARGET_VENDOR       = Microsoft
  endif
  ifeq ($(TARGET_COMPILER), )
    TARGET_COMPILER     = cl
  endif
endif

# MacOS also uses the 'CPU-OS' form. Assume the 'gcc' compiler by default.
ifeq ($(TARGET_OS), macosx)
  ifeq ($(TARGET_VENDOR), unknown)
    TARGET_VENDOR       = Apple
  endif
  ifeq ($(TARGET_COMPILER), )
    TARGET_COMPILER     = gcc
  endif
endif

# Many HALCON Embedded targets use the form 'CPU-VENDOR-OS' instead, with
# the compiler being implicit. Attempt to handle those cases as well (by
# default, assume gcc or a gcc-like compiler).
ifneq ($(filter linux%,$(TARGET_COMPILER)),)
TARGET_VENDOR          := $(word 2,$(subst -, ,$(HALCONARCH)))
TARGET_OS              := $(TARGET_COMPILER)
TARGET_COMPILER         = gcc
else ifneq ($(filter wince%,$(TARGET_COMPILER)),)
TARGET_VENDOR          := $(word 2,$(subst -, ,$(HALCONARCH)))
TARGET_OS              := $(TARGET_COMPILER)
TARGET_COMPILER        := cl
endif

ifeq ($(OS),Windows_NT)
  # Attempt to detect when running in Cygwin on Windows by checking for
  # ComSpec, which should only be set when running from the standard
  # Windows command shell. Checking Cygwin's OSTYPE would be better, but
  # unfortunately this variables is not exported to make by default.
  ifeq ($(ComSpec),)
    OS := cygwin
  endif
else
  # Set the OS to the operating system we are building on (Darwin for OSX,
  # and Linux for Linux systems), analogous to Windows.
  OS := $(shell uname)
endif

# Define some variables to help us deal with the differences between
# Unix and Windows build platforms. (This assumes that 'OS' will be set
# to Windows_NT on all Windows systems).
ifeq (Windows_NT,$(OS))
DIRSEP                 := $(strip \)
ESCDIRSEP              := $(strip \\)
PATHSEP                := ;
MKDIR                  := mkdir
RMDIR                  := rmdir /S /Q
RM                     := del /F /Q
CP                     := copy /Y

# GNU make can't deal with '\' in the wildcard function, but it does work
# if '/' is used as the directory separator instead. To not have to worry
# about this, use $(call WILDCARD,$(PATH),$(PATTERN)) instead of
# $(wildcard $(PATH)$(PATTERN).
WILDCARD                = $(subst /,\,$(wildcard $(subst \,/,$(1))$(2)))

# Copy a file. See the Unix definition below as to why this macro is necessary.
COPYFILE               := copy /Y $1 $2

else
DIRSEP                 := /
ESCDIRSEP              := /
PATHSEP                := :
MKDIR                  := mkdir -p
RMDIR                  := rmdir -v
RM                     := rm -rfv
CP                     := cp

WILDCARD                = $(wildcard $(1)$(2))

# On most systems, 'cp -p' will truncate the timestamp to whatever resolution
# of file modification times 'cp' actually supports, confusing make's logic
# to determine what needs to be remade or not. This macro will work around this
# problem by adjusting the timestamp to 1s later than the original file. Use it
# like this: $(call COPYFILE,<src>,<dst>).
ifeq ($(OS), Darwin)
COPYFILE               := cp -p $1 $2 && touch -A'01' -m $2
else
COPYFILE               := cp -p $1 $2 && \
                          touch -d "$$(date -R -r $2) +1 second" -m $2
endif

endif

# Macro command to escape spaces in paths (required because spaces in paths
# will confuse (GNU)make
EMPTY                  :=
SPACE                  := $(EMPTY) $(EMPTY)
escape_path             = $(subst $(SPACE),\$(SPACE),$(1))
unescape_path           = $(subst \$(SPACE),$(SPACE),$(1))

# Directory definitions
ifeq ($(STATE),debug)
BINDIR                 := bind$(DIRSEP)$(HALCONARCH)
GENDIR                 := gend$(DIRSEP)$(HALCONARCH)
LIBDIR                 := libd$(DIRSEP)$(HALCONARCH)
OBJDIR                 := objd$(DIRSEP)$(HALCONARCH)
else ifeq ($(STATE),tuning)
BINDIR                 := bint$(DIRSEP)$(HALCONARCH)
GENDIR                 := gent$(DIRSEP)$(HALCONARCH)
LIBDIR                 := libt$(DIRSEP)$(HALCONARCH)
OBJDIR                 := objt$(DIRSEP)$(HALCONARCH)
else
BINDIR                 := bin$(DIRSEP)$(HALCONARCH)
GENDIR                 := gen$(DIRSEP)$(HALCONARCH)
LIBDIR                 := lib$(DIRSEP)$(HALCONARCH)
OBJDIR                 := obj$(DIRSEP)$(HALCONARCH)
endif

# Pattern definitions for rules. For GNU make, in pattern on the left hand side
# of a rule, '\%' is taken to be a literal '%', so we must escape the backslash
# on Windows. However, this will not work on the right hand side of a rule,
# so we must only ever use these on the right hand side of a rule.
BINPAT                 := $(BINDIR)$(ESCDIRSEP)%
GENPAT                 := $(GENDIR)$(ESCDIRSEP)%
LIBPAT                 := $(LIBDIR)$(ESCDIRSEP)%
OBJPAT                 := $(OBJDIR)$(ESCDIRSEP)%

# Path definitions. These include the $(DIRSEP) at the end to make the
# resulting strings shorter and easier to read.
BINPATH                := $(BINDIR)$(DIRSEP)
GENPATH                := $(GENDIR)$(DIRSEP)
LIBPATH                := $(LIBDIR)$(DIRSEP)
OBJPATH                := $(OBJDIR)$(DIRSEP)
