// HDevEngine/C++ example
//
// (C) 2018 MVTec Software GmbH
//
// Purpose:
// Count pips on die.
//

#include "HalconCpp.h"
#include "HDevThread.h"
#include "my_hdevoperatorimpl.h"
#include "my_error_output.h"
#include <string>

#include <iostream>
#include <csignal>

// uncomment the following #define for printing the full error description
// instead of the message only
#define FULL_ERR_MSG

using namespace HalconCpp;
using namespace HDevEngineCpp;
using namespace std;

static volatile bool keepRunning = true;

void InitializeProgram(HTuple &acq_handle);
void CountDiePips(HTuple &acq_handle, HString &output_text);
void signalHandler(int signum);
void run(void);

// external procedure: initialize image acquisition
void InitializeProgram(HTuple &acq_handle)
{
  // Create a procedure from an external procedure file
  HDevProcedure proc("init_program");

  // Create a procedure call
  HDevProcedureCall proc_call(proc);

  // Execute the procedure
  proc_call.Execute();

  // Get the results of the init_program procedure
  HTuple result;
  proc_call.GetOutputCtrlParamTuple("AcqHandle", &acq_handle);
}

// external procedure: image processing
void CountDiePips(HDevProcedure &proc, HTuple &acq_handle, HString &output_text)
{

  // Create a procedure call
  HDevProcedureCall proc_call(proc);

  // Set the input parameters of the procedure
  proc_call.SetInputCtrlParamTuple("AcqHandle", acq_handle);

  // Execute the procedure
  proc_call.Execute();

  // Get the output parameters
  output_text = proc_call.GetOutputCtrlParamTuple(1);
}

void signalHandler(int signum)
{
  keepRunning = false;
  //exit(signum);
}

void run(void)
{
  try
  {
    try
    {
      cout << "Loading procedures... \n";
      // Define path of main program and external procedures
      std::string ext_proc_path = "./hdevelop_procedures";
      // Set external procedure path
      HDevEngine().SetProcedurePath(ext_proc_path.c_str());

      // Activate debugging
      HDevEngine().SetEngineAttribute("debug_port", 57786);
      HDevEngine().SetEngineAttribute("debug_password", "workshop");
      HDevEngine().StartDebugServer();

      // Call procedures

      // init_program is an external procedure
      //   -> initialize image acquisition
      HTuple acq_handle;
      cout << "Initializing image acquisition...\n";
      InitializeProgram(acq_handle);

      // Create a procedure from the program and the local procedure name
      HDevProcedure proc("count_die_pips");
      while (keepRunning)
      {
        HString output_text;
        CountDiePips(proc, acq_handle, output_text);
        printf(output_text);
        printf("\n");
      }
      // Cleanup
      CloseFramegrabber(acq_handle);
      cout << "Application stopped.\n";
    }

    catch (HOperatorException hoperatorexception)
    {
      fprintf(stdout, "  Error #%u in %s: %s\n",
              hoperatorexception.ErrorCode(),
              (const char *)hoperatorexception.ProcName(),
              (const char *)hoperatorexception.ErrorMessage());
    }
  }
  catch (HDevEngineException &hdev_exception)
  {
    // If an exception occurs during loading the procedure
    // -> print error message and exit
#ifdef FULL_ERR_MSG
    DispErrorMessage(hdev_exception);
#else
    DispMessage(hdev_exception.Message());
#endif
    exit(SIGINT);
  }
  //exit(SIGINT);
}

int main(int argc, char *argv[])
{
  cout << "Starting program...\n";
  try
  {
    signal(SIGINT, &signalHandler);
    run();
  }
  catch (HException &exception)
  {
    fprintf(stdout, "  Error #%u in %s: %s\n",
            exception.ErrorCode(),
            (const char *)exception.ProcName(),
            (const char *)exception.ErrorMessage());
    return -1;
  }
  return 0;
}
